import java.lang.annotation.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * Clase de utileria que utiliza Reflection
 * 
 * @author G. Cherencio 
 * @version 1.0
 */
public class DoReflection {
    private static HashMap<String,Class<?>> map = new HashMap<String,Class<?>>();
    static {
       map.put("int", Integer.class );
       map.put("long", Long.class );
       map.put("double", Double.class );
       map.put("float", Float.class );
       map.put("bool", Boolean.class );
       map.put("char", Character.class );
       map.put("byte", Byte.class );
       map.put("void", Void.class );
       map.put("short", Short.class );        
    }

    /**
     * Devuelve una referencia de tipo Class o bien null
     * 
     * @param  sc   nombre de una clase
     * @return     referencia Class de clase o bien null
     */
    public static Class<?> getClass(final String sc) {
        Class<?> c = null;
        try {
            c = Class.forName(sc);
        } catch(ClassNotFoundException cnfe) {
            System.err.println("DoReflection.getClass(): Class ["+sc+"] doesnt exist");
        }
        return c;
    }
    public static Object newObject(final Class<?> sc) {
        try {
            return sc.newInstance();
        } catch(InstantiationException ie) {
            return null;
        } catch(IllegalAccessException iae) {
            return null;
        }
    }
    public static int getAnnotationJColumnCount(final Class<?> sc) {
        int n=0;
        try {
            for(Method m : sc.getMethods() ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        n++;
                    }
                }
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getMethodColumn(): SecurityException!");
        }
        return n;
    }
    public static String getQueryLoad(final Class<?> sc) {
        for(Annotation a : sc.getAnnotations()) {
            if ( a instanceof JTableColumn ) {
                return ((JTableColumn) a).queryLoad();
            }
        }
        return "";
    }
    /**
     * devuelve un arreglo de metodos (ordenado por position indicada en JColumn) con los 
     * metodos anotados por JColumn
     */
    public static Method[] getMethodColumn(final Class<?> sc) {
        int n=getAnnotationJColumnCount(sc);
        Method[] met = new Method[n];
        Method[] rmet = new Method[n];
        int[] pos = new int[n];
        int i=0;
        int maxpos=0;
        for(i=0;i<n;i++) {
            met[i]=null;rmet[i]=null;pos[i]=0;
        }
        i=0;
        try {
            for(Method m : sc.getMethods() ) {
                boolean ok=false;
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        ok=true;
                        pos[i]=((JColumn) a).position();
                        if ( pos[i] > maxpos ) maxpos = pos[i];
                        break;
                    }
                }
                if ( ok ) {
                    met[i]=m;
                    i++;
                }
            }
            for(i=0;i<=maxpos;i++) {
                for(int ii=0;ii<n;ii++) {
                    if ( pos[ii] == i ) {
                        rmet[i] = met[ii];
                        break;
                    }
                }
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getMethodColumn(): SecurityException!");
        }
        return  rmet;
    }
    
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static Class<?>[] getColumnClass(Method[] met) {
        Class[] c = new Class[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i] = ((JColumn) a).classType();
                        if ( c[i] == Void.class ) { // no indico clase, asumo la retornada por el metodo o bien String
                            Class<?> x = map.get(m.getReturnType().getName());
                            if ( x == null ) c[i] = String.class;
                            else c[i] = x;
                        }
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnClass(): SecurityException!");
        }
        return c;
    }
    /**
     * Metodo generico para obtener vector de propiedades string1
     * de las columnas, segun lo anotado con JColumn
     */
    public static String[] getColumnString(Method[] met,String methodName) {
        String[] c = new String[met.length];
        for(int i=0;i<met.length;i++) c[i]=null;
        int i=0;
        try {
            methodName = methodName.toLowerCase();
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if ( methodName.equals("title") ) {
                            c[i]=((JColumn) a).title();
                        } else if ( methodName.equals("sqlquery") ) {
                            c[i]=((JColumn) a).sqlQuery();
                        }
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnInt(): SecurityException!");
        }
        return c;
    }
    /**
     * Metodo generico para obtener vector de propiedades enteras
     * de las columnas, segun lo anotado con JColumn
     */
    public static int[] getColumnInt(Method[] met,String methodName) {
        int[] c = new int[met.length];
        for(int i=0;i<met.length;i++) c[i]=-1;
        int i=0;
        try {
            methodName = methodName.toLowerCase();
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if ( methodName.equals("width") ) {
                            c[i]=((JColumn) a).width();
                        } else if ( methodName.equals("position") ) {
                            c[i]=((JColumn) a).position();
                        } else if ( methodName.equals("sqlquerycolumn") ) {
                            c[i]=((JColumn) a).sqlQueryColumn();
                        }
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnInt(): SecurityException!");
        }
        return c;
    }
    /**
     * Metodo generico para obtener vector de propiedades double
     * de las columnas, segun lo anotado con JColumn
     */
    public static double[] getColumnDouble(Method[] met,String methodName) {
        double[] c = new double[met.length];
        for(int i=0;i<met.length;i++) c[i]=-1.0;
        int i=0;
        try {
            methodName = methodName.toLowerCase();
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if (methodName.equals("minrange")) {
                            c[i]=((JColumn) a).minRange();
                        } else if (methodName.equals("maxrange")) {
                            c[i]=((JColumn) a).maxRange();
                        } 
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnInt(): SecurityException!");
        }
        return c;
    }
    
    /**
     * Metodo generico para obtener vector bidimensional de propiedades double
     * de las columnas, segun lo anotado con JColumn
     */
    public static double[][] getColumnListDouble(Method[] met,String methodName) {
        double[][] c = new double[met.length][];
        double[] dv;
        int i=0;
        try {
            methodName = methodName.toLowerCase();
            for(Method m : met ) {
                boolean dimensione=false;
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if (methodName.equals("listrange")) {
                            dv=((JColumn) a).listRange();
                            c[i] = new double[dv.length];
                            dimensione=true;
                            for(int j=0;j<dv.length;j++) c[i][j] = dv[j];
                        } 
                        break;
                    }
                }
                if (!dimensione) c[i]=new double[0];
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnInt(): SecurityException!");
        }
        return c;
    }
    
    /**
     * Metodo generico para obtener vector bidimensional de propiedades String
     * de las columnas, segun lo anotado con JColumn
     */
    public static String[][] getColumnListString(Method[] met,String methodName) {
        String[][] c = new String[met.length][];
        String[] dv;
        int i=0;
        try {
            methodName = methodName.toLowerCase();
            for(Method m : met ) {
                boolean dimensione=false;
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if (methodName.equals("liststring")) {
                            dv=((JColumn) a).listString();
                            c[i] = new String[dv.length];
                            dimensione=true;
                            for(int j=0;j<dv.length;j++) c[i][j] = new String(dv[j]);
                        } 
                        break;
                    }
                }
                if (!dimensione) c[i] = new String[0];
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnInt(): SecurityException!");
        }
        return c;
    }
   
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static int[] getColumnWidth(Method[] met) {
        int[] c = new int[met.length];
        for(int i=0;i<met.length;i++) c[i]=-1;
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).width();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnWidth(): SecurityException!");
        }
        return c;
    }
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static String[] getColumnTitle(Method[] met) {
        String[] c = new String[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).title();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnTitle(): SecurityException!");
        }
        return c;
    }
    /**
     * Metodo generico para obtener vector de propiedades booleanas
     * de las columnas, segun lo anotado con JColumn
     */
    public static boolean[] getColumnBoolean(Method[] met,String methodName) {
        boolean[] c = new boolean[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                methodName = methodName.toLowerCase();
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        if ( methodName.equals("primarykey") ) {
                            c[i]=((JColumn) a).primaryKey();
                        } else if (methodName.equals("rangevalidation")) {
                            c[i]=((JColumn) a).rangeValidation();
                        } else if (methodName.equals("sqlqueryemptyrow")) {
                            c[i]=((JColumn) a).sqlQueryEmptyRow();
                        } else if (methodName.equals("listvalidation")) {
                            c[i]=((JColumn) a).listValidation();
                        } else if (methodName.equals("liststringvalidation")) {
                            c[i]=((JColumn) a).listStringValidation();
                        }
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnBoolean(): SecurityException!");
        }
        return c;
    }

    
    
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static boolean[] getColumnPrimaryKey(Method[] met) {
        boolean[] c = new boolean[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).primaryKey();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getColumnPrimaryKey(): SecurityException!");
        }
        return c;
    }

    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static String[] getSqlQuery(Method[] met) {
        String[] c = new String[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).sqlQuery();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getSqlQuery(): SecurityException!");
        }
        return c;
    }
    
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static int[] getSqlQueryColumn(Method[] met) {
        int[] c = new int[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).sqlQueryColumn();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getSqlQuery(): SecurityException!");
        }
        return c;
    }
    /**
     * A partir del arreglo ordenado de metodos, obtengo los otros arreglos de propiedades
     * de las columnas, segun lo anotado con JColumn
     */
    public static boolean[] getSqlQueryEmptyRow(Method[] met) {
        boolean[] c = new boolean[met.length];
        int i=0;
        try {
            for(Method m : met ) {
                for(Annotation a : m.getDeclaredAnnotations()) {
                    if ( a instanceof JColumn ) {
                        c[i]=((JColumn) a).sqlQueryEmptyRow();
                        break;
                    }
                }
                i++;
            }
        } catch(SecurityException se) {
            System.err.println("DoReflection.getSqlQuery(): SecurityException!");
        }
        return c;
    }
    
    /**
     * Testeo implementacion de interfase
     * 
     * @param  c   una instancia de tipo Class que representa el objeto a analizar
     * @param  sInterface nombre completo (incluye package) de la interfase que se pretende
     *         testear si c implementa o no
     * @return     true si c implementa sInterface, caso contrario, devuelve false 
     */
    public static boolean hasInterface(Class<?> c,final String sInterface) {
        return hasInterface(c.getName(),sInterface);
    }
    
    /**
     * Testeo implementacion de interfase
     * 
     * @param  sc   nombre completo de una clase (incluye package) que representa el objeto a analizar
     * @param  sInterface nombre completo (incluye package) de la interfase que se pretende
     *         testear si c implementa o no
     * @return     true si c implementa sInterface, caso contrario, devuelve false 
     */
    public static boolean hasInterface(final String sc,final String sInterface) {
        boolean implementaProcesable = false;
        try {
            Class c = Class.forName(sc);
            for(Class cint : c.getInterfaces()) {  // si se usa c.getInterfaces() no ubica interfases implementadas en superclases
                if ( cint.getName().equals(sInterface) ) {
                    implementaProcesable = true;
                    break;
                }
            }
        } catch(ClassNotFoundException cnfe) {
            System.err.println("DoReflection.hasInterface(): Class ["+sc+"] doesnt exist");
        }
        return implementaProcesable;
    }
    
    public static int getPageSize(final Class<?> sc) {
        for(Annotation a : sc.getAnnotations()) {
            if ( a instanceof JTableColumn ) {
                return ((JTableColumn) a).pageSize();
            }
        }
        return -1;
    }
    
    public static String getSpPageCount(final Class<?> sc) {
        for(Annotation a : sc.getAnnotations()) {
            if ( a instanceof JTableColumn ) {
                return ((JTableColumn) a).spPageCount();
            }
        }
        return "";
    }
    
    public static boolean getPagination(final Class<?> sc) {
        for(Annotation a : sc.getAnnotations()) {
            if ( a instanceof JTableColumn ) {
                return ((JTableColumn) a).pagination();
            }
        }
        return false;
    }
}
