import java.io.*;
import java.util.*;
import java.text.*;
/**
 * Jugando con fechas. Este programa es interactivo, en modo consola y permite seleccionar las distintas combinaciones
 * de formatos de fecha que podemos tener en Java y ver el resultado de "formatear" la fecha actual en dicho formato.
 * Este programa trabaja con los formatos de fecha predefinidos en Java.
 * Tambien permite ver la ejecucion de un ciclo for infinito:  for(;;) y una salida no estructurada del mismo utilizando
 * break acorde a lo ingresado por el usuario.
 * Para los ingresos por teclado se utiliza el metodo leerLinea().
 * El menu de opciones que se despliega al usuario se realiza en el metodo menu().
 * El metodo main() solo crea una instancia de esta clase, toda la accion se lleva a cabo en el constructor de esta clase.
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Fechas
{
    /**
     * Constructor for objects of class Fechas
     */
    public Fechas()
    {
        for(;;) {
        // menu de opciones
        int tipofecha = menu("Elija el formato de Fecha","Ingrese Opcion:",new String[] 
                                                                         {"1. SHORT",
                                                                          "2. MEDIUM",
                                                                          "3. LONG",
                                                                          "4. FULL",
                                                                          "9. SALIR!!!"});
        if ( tipofecha == 9 ) break; // salida no estructurada                                                                          
        int[] dftipo = new int[] { DateFormat.SHORT, DateFormat.MEDIUM, DateFormat.LONG, DateFormat.FULL };
        Locale[] ubtipo = new Locale[] { Locale.CANADA, Locale.CANADA_FRENCH, Locale.CHINA, Locale.CHINESE, Locale.ENGLISH,
                                         Locale.FRANCE, Locale.FRENCH, Locale.GERMAN, Locale.GERMANY, Locale.ITALIAN, Locale.ITALY,
                                         Locale.JAPAN, Locale.JAPANESE, Locale.KOREA, Locale.KOREAN, Locale.PRC, Locale.SIMPLIFIED_CHINESE,
                                         Locale.TAIWAN, Locale.TRADITIONAL_CHINESE, Locale.UK, Locale.US };
        int ubicacion = menu("Elija el formato de Fecha","Ingrese Opcion:",new String[] 
                                                                         {"01. CANADA",
                                                                          "02. CANADA FRENCH",
                                                                          "03. CHINA",
                                                                          "04. CHINESE",
                                                                          "05. ENGLISH",
                                                                          "06. FRANCE",
                                                                          "07. FRENCH",
                                                                          "08. GERMAN",
                                                                          "09. GERMANY",
                                                                          "10. ITALIAN",
                                                                          "11. ITALY",
                                                                          "12. JAPAN",
                                                                          "13. JAPANESE",
                                                                          "14. KOREA",
                                                                          "15. KOREAN",
                                                                          "16. PRC",
                                                                          "17. SIMPLIFIED CHINESE",
                                                                          "18. TAIWAN",
                                                                          "19. TRADITIONAL CHINESE",
                                                                          "20. UK",
                                                                          "21. US",
                                                                          "99. SALIR!!!"});
        if ( ubicacion == 99 ) break; // salida no estructurada                                                                          
        Date hoy = new Date();
        Date ayer = new Date();
        ayer.setTime(ayer.getTime()-(24L*60L*60L*1000L));
        DateFormat df = DateFormat.getDateInstance(dftipo[tipofecha-1],ubtipo[ubicacion-1]);
        System.out.println("//////////////////////////////////////////////////////////////////////////////");
        System.out.println("Fecha Actual en formato elegido: " + df.format(hoy));
        System.out.println("Fecha de Ayer en formato elegido: " + df.format(ayer));
        System.out.println("Pasaron " + hoy.getTime()+" milisegundos desde 01/01/1970");
        System.out.println("//////////////////////////////////////////////////////////////////////////////");
        obtenerOpcion("Presione Intro para continuar");
        }
        System.out.println("Fin Ejecucion de Programa");
    }
    /**
     * Incio de ejecucion del programa, creo una instancia de la clase Fechas
     * 
     * @args  parametros indicados en la linea de comando
     */
    public static void main(String args[])
    {
        new Fechas();
    }
    /**
     * Este metodo lee una linea de la standard input = STDIN = System.in = teclado
     * y devuelve dicha linea en forma de String.
     * Toma System.in un stream de tipo InputStream (8 bits,de input), a traves de
     * InputStreamReader lo transforma en un stream de 16 bits, de input, que, a su vez,
     * es manejado en forma buffered (para acumular los caracteres tipeados y leerlos todos juntos) con
     * BufferedReader (stream buffered, 16 bits, input) a traves de su metodo .readLine()
     * @return String conteniendo los caracteres ingresados por teclado
     */
    private String leerLinea() {
        String linea = "";
        try {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in)); //IOE
            linea = in.readLine();
        } catch(IOException ioe) {
            System.err.println("leerLinea(): Error leyendo linea de teclado " + ioe);
        }
        return linea;
    }
    /**
     * Este metodo genera un menu de tipo consola, indicando un titulo para el mismo,
     * presenta un "prompt" al usuario para el ingreso de la opcion elegida y utiliza
     * un arreglo de tipo String para indicar cada una de las opciones a elegir.
     * Cada opcion tiene el formato: xxx.yyyyyyyyyyyyyyyyyyyyyy  donde xxx significa el
     * numero de opcion luego le sigue un punto y luego el resto de la descripcion de la
     * opcion.
     * Controla que el ingreso del usuario se corresponda con alguna de las opciones indicadas
     * en las opciones del menu.
     * @param titulo Titulo del menu
     * @param prompt Prompt del usuario para hacer ingreso de opcion a elegir
     * @param opciones Arreglo de tipo String con c-u de las opciones del menu a desplegar con formato xxx.yyyyyyyyyyyyyy
     * @return Devuelve el nro de opcion seleccionada en el menu o bien -1 si hubo algun error
     */
    public int  menu(String titulo,String prompt,String[] opciones) {
        String str = "";
        boolean opcionOk = false;
        int largo = 0;
        int retorno = -1;
        if ( opciones.length == 0 ) return retorno;  // lista de opciones vacia
        largo = opciones[0].indexOf('.');
        if ( largo == -1 || largo == 0 ) return retorno; // no se encontro '.' o bien era el primer caracter 
        do {
            System.out.println(titulo);
            for(int i = 0;i<opciones.length;i++) System.out.println(opciones[i]);
            str = obtenerOpcion(prompt); // obtengo ingreso
            if ( str.length() == largo ) {
                for(int i = 0;i<opciones.length && !opcionOk;i++) {  // verifico ingreso contra opciones
                   boolean ok = true;
                   for(int w = 0;w<largo && ok;w++)
                      ok = (str.charAt(w) == opciones[i].charAt(w));
                   opcionOk = ok;
               }
            }
        } while (!opcionOk);
        // paso str a int
        try {
            Integer i = new Integer(str);
            retorno = i.intValue();
        } catch(NumberFormatException nfe) {
        }
        return retorno;
    }
    /**
     * Este metodo presenta un "prompt" al usuario para el ingreso de caracteres
     * por teclado, los caracteres tipeados (hasta presionar Intro) son devueltos
     * en forma de String.
     * @param prompt Prompt del usuario para hacer ingreso por teclado
     * @return Caracteres tipeados en forma de String
     */
    private String obtenerOpcion(String prompt) {
        System.out.print(prompt);
        return leerLinea();
    }
}
