import java.io.*;
import java.util.*;
import java.text.*;
/**
 * Jugando con fechas (formatos custom de fechas).
 * Este programa es interactivo, en modo consola y permite ingresar un formato de fecha, acorde a la
 * informacion contenida en el archivo de texto fechas.txt que es mostrado en pantalla, para que el 
 * usuario tenga ayuda de como armar "patterns" (formatos) para formatos de fecha "custom" (a medida).
 * Este programa trabaja con los formatos de fecha "no predefinidos" en Java, trabaja con formatos 
 * "customizados" acorde a la necesidad que tengamos.
 * 
 * Ejemplos de formatos ingresados y salidas obtenidas
 * dd/MM/yyyy                               01/10/2004
 * dd 'de' MMMMMMMM 'del' yyyy              01 de Octubre del 2004
 * dd/MM/yyyy hh:mm:ss a                    01/10/2004 10:52:01 AM
 * EEEEEEEEE, EEE, MMM, MMMMMMMMM, d, dd    Viernes, Vie, Oct, Octubre, 1, 01
 * 
 * Tambien permite ver la ejecucion de un ciclo for for(;<expr condicional>;) y una salida estructurada 
 * del mismo.
 * Para los ingresos por teclado se utiliza el metodo leerLinea().
 * El metodo mostrarArchivo() muestra el contenido de fechas.txt en la consola.
 * El menu de opciones que se despliega al usuario se realiza en el metodo menu().
 * El metodo main() solo crea una instancia de esta clase, toda la accion se lleva a cabo en el constructor
 * de esta clase.
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Fechas2
{
    /**
     * Constructor for objects of class Fechas
     */
    public Fechas2()
    {
        String formato,fecha_custom;
        // tomos los simbolos para formato de fechas en ingles, US
        DateFormatSymbols simbolosFecha = new DateFormatSymbols(new Locale("en","US")); 
        String[] nombreDeDia = {
            "", "Domingo", "Lunes", "Martes", "Miercoles", "Jueves", "Viernes", "Sabado"};
        String[] nombreCortoDeDia = {
            "", "Dom", "Lun", "Mar", "Mie", "Jue", "Vie", "Sab"};
        String[] nombreDeMes = {
            "Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", 
            "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre"};
        String[] nombreCortoDeMes = {
             "Ene", "Feb", "Mar", "Abr", "May", "Jun", "Jul", "Ago", "Sep", "Oct", "Nov", "Dic"};
        // cambio los nombres de los meses y dias en ingles por los nuestros en castellano
        simbolosFecha.setMonths(nombreDeMes);
        simbolosFecha.setWeekdays(nombreDeDia);
        simbolosFecha.setShortMonths(nombreCortoDeMes);
        simbolosFecha.setShortWeekdays(nombreCortoDeDia);
        
        // fecha del dia a formatear
        Date hoy = new Date();
        
        boolean continuar = true;
        for(;continuar;) {
        // menu de opciones
        mostrarArchivo("fechas.txt");
        formato = obtenerOpcion("Recomiendo que en la terminal de BlueJ, active las opciones:\n" +
                                "Clear Screen at method call\n" +
                                "Unlimited Buffering\n" + 
                                "Arme su formato \"customizado\" de Fecha:");
                                
        SimpleDateFormat formatoFecha = new SimpleDateFormat(formato,simbolosFecha);

        System.out.println("//////////////////////////////////////////////////////////////////////////////");
        System.out.println("Fecha Actual en formato elegido: " + formatoFecha.format(hoy));
        // parsing de una fecha determinada (en vez de usar la fecha de hoy)
        fecha_custom = obtenerOpcion("Ingrese una Fecha Valida Cualquiera en Formato " + formato + ": ");
        Date fecha = null;
        try { 
            fecha  = formatoFecha.parse(fecha_custom,new ParsePosition(0));
            if ( fecha != null ) {
                System.out.println("Puede formatear la fecha " + fecha_custom + " y crear objeto Date con el formato " + formato);
                System.out.println("Fecha creada: " + fecha);
            } else {
                System.out.println("No puede formatear la fecha " + fecha_custom + " con el formato " + formato);
            }
        } catch(NullPointerException npe) {
            System.out.println("No puede formatear la fecha " + fecha_custom + " con el formato " + formato);
        }
        System.out.println("//////////////////////////////////////////////////////////////////////////////");

        continuar = (menu("�Continuo Formateando Fechas?","Ingrese Opcion:",new String[] 
                                                                         {"1. Continuo",
                                                                          "2. Me Voy!"}) == 1);
        }
        System.out.println("Fin Ejecucion de Programa");
    }
    /**
     * Incio de ejecucion del programa, creo una instancia de la clase Fechas
     * 
     * @args  parametros indicados en la linea de comando
     */
    public static void main(String args[])
    {
        new Fechas2();
    }
    /**
     * Este metodo lee una linea de la standard input = STDIN = System.in = teclado
     * y devuelve dicha linea en forma de String.
     * Toma System.in un stream de tipo InputStream (8 bits,de input), a traves de
     * InputStreamReader lo transforma en un stream de 16 bits, de input, que, a su vez,
     * es manejado en forma buffered (para acumular los caracteres tipeados y leerlos todos juntos) con
     * BufferedReader (stream buffered, 16 bits, input) a traves de su metodo .readLine()
     * @return String conteniendo los caracteres ingresados por teclado
     */
    private String leerLinea() {
        String linea = "";
        try {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in)); //IOE
            linea = in.readLine();
        } catch(IOException ioe) {
            System.err.println("leerLinea(): Error leyendo linea de teclado " + ioe);
        }
        return linea;
    }
    /**
     * Este metodo genera un menu de tipo consola, indicando un titulo para el mismo,
     * presenta un "prompt" al usuario para el ingreso de la opcion elegida y utiliza
     * un arreglo de tipo String para indicar cada una de las opciones a elegir.
     * Cada opcion tiene el formato: xxx.yyyyyyyyyyyyyyyyyyyyyy  donde xxx significa el
     * numero de opcion luego le sigue un punto y luego el resto de la descripcion de la
     * opcion.
     * Controla que el ingreso del usuario se corresponda con alguna de las opciones indicadas
     * en las opciones del menu.
     * @param titulo Titulo del menu
     * @param prompt Prompt del usuario para hacer ingreso de opcion a elegir
     * @param opciones Arreglo de tipo String con c-u de las opciones del menu a desplegar con formato xxx.yyyyyyyyyyyyyy
     * @return Devuelve el nro de opcion seleccionada en el menu o bien -1 si hubo algun error
     */
    public int  menu(String titulo,String prompt,String[] opciones) {
        String str = "";
        boolean opcionOk = false;
        int largo = 0;
        int retorno = -1;
        if ( opciones.length == 0 ) return retorno;  // lista de opciones vacia
        largo = opciones[0].indexOf('.');
        if ( largo == -1 || largo == 0 ) return retorno; // no se encontro '.' o bien era el primer caracter 
        do {
            System.out.println(titulo);
            for(int i = 0;i<opciones.length;i++) System.out.println(opciones[i]);
            str = obtenerOpcion(prompt); // obtengo ingreso
            if ( str.length() == largo ) {
                for(int i = 0;i<opciones.length && !opcionOk;i++) {  // verifico ingreso contra opciones
                   boolean ok = true;
                   for(int w = 0;w<largo && ok;w++)
                      ok = (str.charAt(w) == opciones[i].charAt(w));
                   opcionOk = ok;
               }
            }
        } while (!opcionOk);
        // paso str a int
        try {
            Integer i = new Integer(str);
            retorno = i.intValue();
        } catch(NumberFormatException nfe) {
        }
        return retorno;
    }
    /**
     * Este metodo presenta un "prompt" al usuario para el ingreso de caracteres
     * por teclado, los caracteres tipeados (hasta presionar Intro) son devueltos
     * en forma de String.
     * @param prompt Prompt del usuario para hacer ingreso por teclado
     * @return Caracteres tipeados en forma de String
     */
    private String obtenerOpcion(String prompt) {
        System.out.print(prompt);
        return leerLinea();
    }
    /**
     * Este metodo lee un archivo de texto y muestra su contenido en pantalla
     * 
     * @param archivo Nombre (pathname) del archivo a ser mostrado en pantalla
     */
    public void mostrarArchivo(String archivo) {
        try {
            BufferedReader in = new BufferedReader(new FileReader(archivo));
            String linea;
            try {
                // leo linea de texto del archivo y la muestro en pantalla
                while ( (linea = in.readLine()) != null ) System.out.println(linea);
            } catch(IOException ioe) {
                System.out.println("IOException 2 leyendo archivo " + archivo + "\n" + ioe );
            } finally {
                try { in.close(); 
                } catch(IOException ioe) { 
                    System.out.println("IOException 3 cerrando archivo " + archivo + "\n" + ioe); 
                }
            }
        } catch(IOException ioe) {
            System.out.println("IOException 1 intentando abrir archivo " + archivo + "\n" + ioe );
        }
    }
}
