
import java.io.*;
/**
 * Clase con metodos static para el ingreso de datos
 * por teclado
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Consola
{


    /**
     * Devuelve una linea leida del teclado
     * 
     * @return     linea ingresada por teclado
     */
    public static String ingresoLinea() {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));        
        String ret = "";
        try {
            ret = in.readLine();
        } catch(IOException ioe) {
            System.err.println("Error ingresando linea:\n"+ioe.getMessage());
        }
        return ret;
    }
    /**
     * Devuelve una linea leida del teclado
     * Primero imprime prompt para que el usuario sepa de que se trata este ingreso por teclado
     * 
     * @return     linea ingresada por teclado
     */
    public static String ingresoLinea(String prompt) {
        System.out.println(prompt);
        return ingresoLinea();
    }
    
    public static boolean ingresoSiNo(String prompt) {
        String srta = ingresoLinea(prompt);
        if ( srta.equalsIgnoreCase("S") ) srta = "true";
        else srta = "false";
        Boolean b = new Boolean(srta);
        return b.booleanValue();
    }
    
    /**
     * Devuelve un int ingresado por teclado
     * 
     * @return     entero ingresado por teclado
     */
    public static int ingresoInt() {
        Integer i=null;
        boolean reitero;
        String linea=null;
        do {
            reitero=false;
            try {
                linea = ingresoLinea();
                i = new Integer(linea);
            } catch(NumberFormatException nfe) {
                System.err.println("["+linea+"] no es un entero!");
                reitero=true;
            }
        } while(reitero);
        return i;
    }
    /**
     * Devuelve un int ingresado por teclado
     * Primero imprime prompt
     * 
     * @return     entero ingresado por teclado
     */
    public static int ingresoInt(String prompt) {
        System.out.println(prompt);
        return ingresoInt();
    }
    
    /**
     * Hace un menu de opciones
     * 
     * @return     la opcion ingresada por el usuario
     */
    public static int menu(String[] opc) {
        for(String op : opc) 
            System.out.println(op);
        System.out.println("\nIngrese su Opcion:");
        return ingresoInt();
    }
    
    
}
