import java.io.*;

/**
 * Clase para el manejo de archivo de acceso directo
 * que permite leer y grabar bytes de un archivo de acceso directo
 * 
 * @author G.Cherencio 
 * @version 1.0
 */
public class Directo {
    private String archivo = "directo.dat"; // nombre (x defecto) del archivo
    private int nmaxreg = 1000;  // nro maximo de registros (x defecto)
    private int largreg = 206;  // largo de registro (x defecto)
    private RandomAccessFile raf = null;

    /**
     * Constructor for objects of class Directo
     */
    public Directo() {
        crear();
    }

    public Directo(String f,int mr,int lr) {
        archivo = f;
        nmaxreg = mr;
        largreg = lr;
        crear();
    }

    public void abrir() {
        try {
            raf = new RandomAccessFile(archivo,"rw");
        } catch(FileNotFoundException fnfe) {
            System.err.println("abrir():Error (fnfe) al intentar abrir archivo "+
                archivo+"\n"+fnfe.getMessage());
        }
    }
   
    /**
     * Metodo para crear archivo de acceso directo
     * metodo sobrecargado (overloading)
     */
    public void crear() {
        try {
            File f = new File(archivo);
            if ( f.exists() ) {
                raf = new RandomAccessFile(archivo,"rw");
                return;
            }
            raf = new RandomAccessFile(archivo,"rw");
            // raf con 0 byte
            // declaro y creo un arreglo de getLargoReg() bytes
            byte[] registro = new byte[largreg];
            // lleno de 0 binarios a todo el arreglo
            for(int i=0;i<registro.length;i++) registro[i]=0;
            // grabo N registros en el archivo
            for(int i=0;i<nmaxreg;i++) raf.write(registro);
            // cierro el archivo
            raf.close();
            raf = new RandomAccessFile(archivo,"rw");
        } catch(IllegalArgumentException iae) {
            System.err.println("crear():Error (iae) al intentar crear archivo "+
                archivo+"\n"+iae.getMessage());
        } catch(FileNotFoundException fnfe) {
            System.err.println("crear():Error (fnfe) al intentar crear archivo "+
                archivo+"\n"+fnfe.getMessage());
        } catch(IOException ioe) {
            System.err.println("crear():Error (ioe) al intentar crear archivo "+
                archivo+"\n"+ioe.getMessage());
        }
    }

    
    /**
     * invoco a este metodo cuando ya no quiero trabajar mas con el archivo
     */
    public void cerrar() {
        if ( raf != null ) {
            try {
                // cierro el archivo
                raf.close();
                raf = null;
            } catch(IOException ioe) {
                System.err.println("cerrar():Error (ioe) al intentar cerrar archivo "+
                    archivo+"\n"+ioe.getMessage());
            }
        }
    }

    
    /**
     * destruyo archivo si existia con anterioridad
     */
    public void destruir() {
        if ( raf != null ) cerrar();
        File f = new File(archivo);
        if ( f.exists() ) f.delete();
    }

    
    /**
     * Lee un registro del archivo grabado como una serie de bytes 
     * y lo carga en el objeto actual
     */
    public byte[] leer(int nr) {
        byte[] registro = null;
        if ( raf != null ) {
            if ( nr >= 0 && nr < nmaxreg ) {
                int i=0;
                // el registro es valido
                registro = new byte[largreg];
                // leo registro
                try {
                    // leer el registro
                    // primero me posiciono dentro del archivo de acceso directo
                    raf.seek(nr*largreg);
                    raf.read(registro);

                // el vector registro, contiene todos los bytes que representa a este objeto
                } catch(IOException ioe) {
                    System.err.println("leer():Error (ioe) al intentar leer registro "+ nr + " en archivo "+
                        archivo+"\n"+ioe.getMessage());
                    registro = null;
                }
                
            }
        }
        return registro;
    }

    /**
     * Toma el objeto FAlumno y lo graba en el numero de registro nr
     */
    public void grabar(int nr,byte[] registro) {
System.out.println("llegue a Directo.grabar()");        
        if ( raf != null ) {
            if ( nr >= 0 && nr <= nmaxreg ) {
                if ( registro != null && registro.length == largreg ) {
                    // el vector registro, contiene todos los bytes que representa a este objeto
                    // grabar el registro
                    // primero me posiciono dentro del archivo de acceso directo
                    try {
                        raf.seek(nr*largreg);
                        raf.write(registro);
                        if ( nr == nmaxreg ) nmaxreg++;
                    } catch(IOException ioe) {
                        System.err.println("grabar():Error (ioe) al intentar grabar registro "+ nr + " en archivo "+
                            archivo+"\n"+ioe.getMessage());
                    }
                }
            } // permito grabar mas alla del ultimo
        }
    }
    

    
}
