import java.io.*;
/**
 * FAlumno es una clase POJO standard que puede persistirse
 * en un archivo de acceso directo.
 * La misma clase implementa los metodos para leer y grabar
 * registros en el archivo de acceso directo.
 * Utiliza clase RandomAccessFile
 * 
 * @author G.Cherencio 
 * @version 1.0
 */
public class FAlumno extends FObjectRegistro {
    
    // atributos de instancia
    //private byte estado;   // 1 byte, 0 libre, 1 ocupado, 2 borrado
    //private int legajo;    // 5 bytes
    //private String ape;    // 100 bytes
    //private String nom;    // 100 bytes
    //                      ------
    //                        206 bytes total largo de registro
    
    private Alumno al = null;
    
    /**
     * Constructor for objects of class FAlumno
     */
    public FAlumno() {
        // initialise instance variables
        this(0);
    }
    
    /**
     * Constructor for objects of class FAlumno
     */
    public FAlumno(int l) {
        this(l,"");
    }
    /**
     * Constructor for objects of class FAlumno
     */
    public FAlumno(int l,String ap) {
        this(l,ap,"");
    }
    /**
     * Constructor for objects of class FAlumno
     */
    public FAlumno(int l,String ap,String no) {
        this((byte) 0,l,ap,no);
    }
    /**
     * Constructor for objects of class FAlumno
     */
    public FAlumno(byte es,int l,String ap,String no) {
        al = new Alumno(l,ap,no);
        setEstado(es);
    }
    
    // setters
    // this.estado es la propiedad o atributo estado, 
    // mientras que estado es el argumento estado del metodo setEstado()
    //public void setEstado(byte estado) { this.estado = estado; } 
    public void setLegajo(int l) { al.setLegajo(l); }
    public void setApellido(String a) { al.setApellido(a); }
    public void setNombre(String n) { al.setNombre(n); }

    // getters
    //public byte getEstado() { return estado; }
    public int getLegajo() { return al.getLegajo(); }
    public String getApellido() { return al.getApellido(); }
    public String getNombre() { return al.getNombre(); }
    
    // custom
    /**
     * Toma el objeto FAlumno y lo graba en el numero de registro nr
     */
    @Override
    public boolean grabar(int nr) {
        boolean ret = false;
        setUltimoRegistro(-1);
        if ( getDirecto() != null ) {
System.out.println("llegue a FAlumno grabar() nr="+nr+" maxreg="+getMaxReg());                 
            if ( nr >= 0 && nr < getMaxReg() ) {
System.out.println("llegue a FAlumno grabar() pase validacion!");                 
                setUltimoRegistro(nr);
                int i=0;
                // el registro es valido
                byte[] registro = new byte[getLargoReg()];
                // estado, 1 byte
                //setEstado((byte) 1); // ocupado
                registro[i++]=getEstado();
                // legajo, 5 bytes
                byte[] blegajo = Util.stringToByte(Util.intToString(al.getLegajo(),5),5);
                for(int z=0;z<blegajo.length;z++,i++) registro[i]=blegajo[z];
                // apellido, 100 bytes
                byte[] bape = Util.stringToByte(al.getApellido(),100);
                for(int z=0;z<bape.length;z++,i++) registro[i]=bape[z];
                // nombre, 100 bytes
                byte[] bnom = Util.stringToByte(al.getNombre(),100);
                for(int z=0;z<bnom.length;z++,i++) registro[i]=bnom[z];
                // el vector registro, contiene todos los bytes que representa a este objeto
                // grabar el registro
                getDirecto().grabar(nr,registro);
                
                ret = true;
            }
        }
        return ret;
    }
    
    /**
     * Lee un registro del archivo grabado como una serie de bytes y lo carga en el objeto actual
     */
    @Override
    public boolean leer(int nr) {
        boolean ret = false;
        setUltimoRegistro(-1);
        if ( getDirecto() != null ) {
            if ( nr >= 0 && nr < getMaxReg() ) {
                setUltimoRegistro(nr);
                int i=0;
                // el registro es valido
                // leo registro
                byte[] registro = getDirecto().leer(nr);
                if ( registro != null ) {
                    // estado, 1 byte
                    //setEstado((byte) registro[i++]); // ocupado,libre,borrado
                    setEstado(registro);i++;
                    
                    // legajo, 5 bytes
                    byte[] blegajo = new byte[5];
                    for(int z=0;z<blegajo.length;z++,i++) blegajo[z]=registro[i];
                    String slegajo = new String(blegajo);
                    try {
                        Integer ilegajo = new Integer(slegajo.trim());
                        al.setLegajo(ilegajo);
                    } catch(NumberFormatException nfe) {
                        al.setLegajo(0);
                    }
                    
                    // apellido, 100 bytes
                    byte[] bape = new byte[100];
                    for(int z=0;z<bape.length;z++,i++) bape[z]=registro[i];
                    String sape = new String(bape);
                    al.setApellido(sape); // setApellido(new String(bape));
                    
                    // nombre, 100 bytes
                    byte[] bnom = new byte[100];
                    for(int z=0;z<bnom.length;z++,i++) bnom[z]=registro[i];
                    al.setNombre(new String(bnom));
                    
                    ret=true;
                }
                
            }
        }
        return ret;
    }
    
    /*
     * Representacion String de un objeto FAlumnno
     */
    @Override
    public String toString() {
        return super.toString()+"["+getLegajo()+","+getApellido().trim()+","+getNombre().trim()+"]";
    }
}
