
/**
 * Clase Abstracta, superclase de todas las clases de tipo F*
 * que se van a utilizar para persistir objetos en archivos de
 * acceso directo
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public abstract class FObject {
    // atributos de clase
    private static String archivo = "falumno.dat"; // nombre del archivo
    private static int nmaxreg = 1000;  // nro maximo de registros
    private static int largreg = 206;  // largo de registro
    
    private static Directo di = new Directo(archivo,nmaxreg,largreg);
    

    // setters para atributos de clase
    public static void setArchivo(String a) { archivo = a; }
    public static void setMaxReg(int mr) {
System.out.println("FObject.setMaxReg() mr="+mr);        
        nmaxreg = mr; }
    public static void setLargoReg(int lr) { largreg = lr; }

    // getters para atributos de clase
    public static String getArchivo() { return archivo; }
    public static int getMaxReg() { return nmaxreg; }
    public static int getLargoReg() { return largreg; }
    public static Directo getDirecto() { return di; }

    // custom static
    /**
     * Inicializar al atributo di en base al archivo, largo de registro y nro maximo
     * de registros indicados previamente
     */
    public static void init() {
        if ( di != null ) di.cerrar();
        di = new Directo(getArchivo(),getMaxReg(),getLargoReg());
    }
    public static void init(String a,int mr,int lr) {
        setArchivo(a);setMaxReg(mr);setLargoReg(lr);
        init();
    }
    // ----------------------------------------------
    /**
     * Metodo para crear archivo de acceso directo
     * metodo sobrecargado (overloading)
     */
    public static void crear() {
        crear(getArchivo(),getMaxReg(),getLargoReg());
    }
    /**
     * Metodo para crear archivo de acceso directo
     * metodo sobrecargado (overloading)
     */
    public static void crear(String ar) {
        crear(ar,getMaxReg(),getLargoReg());
    }
    /**
     * Metodo para crear archivo de acceso directo
     * metodo sobrecargado (overloading)
     */
    public static void crear(String ar,int mr) {
        crear(ar,mr,getLargoReg());
    }
    /**
     * Metodo para crear archivo de acceso directo
     * metodo sobrecargado (overloading)
     */
    public static void crear(String a,int mr,int lr) {
        init(a,mr,lr);
        getDirecto().destruir();
        init(a,mr,lr);
    }
    
    /**
     * Metodo para destruir objeto Directo
     * y des-asociar FObject de Directo
     */
    public static void destruir() {
        if ( di != null ) di.destruir();
        di=null;
    }
    
    /**
     * si previamente llame a cerrar o destruir, debo volver a asociar a 
     * Directo con FObject
     */
    public static void abrir() {
        if ( getDirecto() == null ) {
            di = new Directo(getArchivo(),getMaxReg(),getLargoReg());
        }
    }
    
    /**
     * si previamente llame a abrir, puedo des-asociar objeto Directo de FObject 
     */
    public static void cerrar() {
        if ( getDirecto() != null ) {
            di.cerrar();
            di=null;
        }
    }
    
    // custom
    /**
     * Toma el objeto y lo graba en el numero de registro nr
     * cada clase que deriva de esta esta obligada a dar una
     * implementacion de este metodo
     */
    public abstract boolean grabar(int nr);

    /**
     * El objeto actual es reemplazado por el objeto que se encuentra previamente
     * grabado en registro nr
     * Lee un registro del archivo grabado como una serie de bytes y lo carga en el objeto actual
     * cada clase que deriva de esta esta obligada a dar una
     * implementacion de este metodo
     */
    public abstract boolean leer(int nr);    

    
    /*
     * Representacion String de un objeto FObject
     */
    @Override
    public String toString() {
        return "("+getArchivo()+","+getMaxReg()+","+getLargoReg()+")";
    }
    
}
