import java.io.*;
/**
 * Clase que encapsula un objeto RandomAccessFile para implementar
 * un archivo de acceso directo
 * La propiedad rlen indica la longitud de registro
 * La propiedad nreg indica la cantidad maxima de registros
 * 
 * @author G.Cherencio
 * @version 1.0
 */
public class RFile
{
    // instance variables - replace the example below with your own
    private String archivo;
    private int rlen;
    private int nreg;
    private RandomAccessFile raf;

    /**
     * Constructor for objects of class RFile
     */
    public RFile(){
        // asume archivo data.dat con long de registro=256 bytes y 100 registros
        this("data.dat",256,100); 
    }
    public RFile(String f) {
        this(f,256,100);
    }
    public RFile(String f,int rl) {
        this(f,rl,100);
    }
    public RFile(String f,int rl,int n) {
        setArchivo(f);
        setLargoRegistro(rl);
        setNRegistro(n);
        // si no existe el archivo lo genero
        File fi = new File(getArchivo());
        if ( !fi.exists() ) {
            generar();
        } else abrir();
    }
    
    // GETTERS, SETTERS
    public void setArchivo(String s) { archivo = s; }
    public void setLargoRegistro(int rl) { rlen = rl; }
    public void setNRegistro(int n) { nreg = n; }
    
    public String getArchivo() { return archivo; }
    public int getLargoRegistro() { return rlen;}
    public int getNRegistro() { return nreg; }
    
    // CUSTOM
    private void abrir() {
        try {
            raf = new RandomAccessFile(getArchivo(),"rw");
        } catch(IOException ioe) {
            System.err.println("abrir():Error abriendo archivo "+getArchivo()+"\n"+ioe.getMessage());
        }
    }
    
    public void cerrar() {
        try {
            raf.close();
        } catch(IOException ioe) {
            System.err.println("cerrar():Error cerrando archivo "+getArchivo()+"\n"+ioe.getMessage());
        }
    }
    
    public byte[] setValues(byte[] b, byte value) {
        // lleno vector de nulos o valores 0 binario
        for(int i=0;i<b.length;i++) b[i]=value;
        return b;
    }
    
    public void generar() {
        try {
            raf = new RandomAccessFile(getArchivo(),"rw");
            // creo vector de registro de bytes
            byte b[] = new byte[getLargoRegistro()];
            setValues(b,(byte) 0);
            // grabo tantos registros como sea necesario
            for(int i=0;i<getNRegistro();i++) raf.write(b);
            raf.close();
            abrir();
        } catch(IOException ioe) {
            System.err.println("generar():Error creando archivo "+getArchivo()+"\n"+ioe.getMessage());
        }
    }

    public byte[] leer(int nro_registro) {
        byte b[] = new byte[getLargoRegistro()];
        setValues(b,(byte) 0);
        if ( nro_registro < 0 || nro_registro >= getNRegistro() ) return b;
        try {
            raf.seek(nro_registro*getLargoRegistro());
            raf.read(b);
        } catch(IOException ioe) {
            System.err.println("leer():Error leyendo archivo "+getArchivo()+"\n"+ioe.getMessage());
        }
        return b;
    }

    public void grabar(int nro_registro,byte[] b) {
        if ( b.length != getLargoRegistro() ) return;
        if ( nro_registro < 0 || nro_registro >= getNRegistro() ) return;
        try {
            raf.seek(nro_registro*getLargoRegistro());
            raf.write(b);
        } catch(IOException ioe) {
            System.err.println("grabar():Error grabando archivo "+getArchivo()+"\n"+ioe.getMessage());
        }
    }
    
}
