
/**
 * Esta clase no compila!!.
 * Aunque parece estar todo bien, sin embargo, como no hemos implementado ningun constructor,
 * Java provee automaticamente, implicitamente, de un constructor nulo, dicho constructor no
 * tiene parametros ni implementacion, si esta clase no fuese derivada tendria la siguiente forma:
 *                 public Hijo() { }
 * pero como esta clase es derivada, el constructor nulo insertado por Java para una clase derivada,
 * tiene la siguiente forma:
 *                 public Hijo() { super(); }
 * y es aqui donde esta el problema de esta clase, puesto que no existe una implementacion de constructor
 * nulo en la clase Padre (porque Padre implementa un constructor de tipo: public Padre(String s) { ... } )
 * Java nunca insertara un constructor nulo en la clase Padre, porque esta clase ya tiene un constructor
 * no nulo implementado.
 * Como salir de esta situacion?
 * Debemos tener el cuidado de incluir un constructor nulo en la superclase o bien asegurarnos de incluir
 * los constructores correspondientes en la clase derivada a modo de asegurarnos una correcta instanciacion
 * de los objetos de la clase derivada
 * Por ejemplo: si incluimos el siguiente constructor:
 * 
 * public Hijo() { }
 * 
 * esta clase tampoco compilara, porque habra una llamada implicita a super() y no existe constructor nulo en Padre.
 * Entonces, deberiamos implementar un constructor como este:
 * 
 * public Hijo() { super("NN"); }
 * 
 * para que esta clase compile, ya que, el constructor implementado en la clase Padre es: public Padre(String s) { ... }
 * Esto si permitiria que puedan crear instancias de clase Hijo, ya que me asegura que se pueda tambien llamar a un constructor
 * de la clase Padre.
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Hijo extends Padre
{
    private String segundoNombre;
    public Hijo() {
        super("no se");
        setSegundoNombre("");
    }
    public Hijo(String str) {
        super(str);
        setSegundoNombre("");
    }
    public void setSegundoNombre(String s) { segundoNombre = s; }
    /**
     * Creo una instancia de la clase Hijo
     */
    public static void main(String[] args) 
    {
        Hijo h = new Hijo();
        Hijo h2 = new Hijo("Hola mundo");
        System.out.println("nombre="+h.getNombre().length());
        System.out.println("nombre="+h2.getNombre().length());
    }
}
