/**
 * Este programa permite observar:<br>
 * Uso de bloques de inicializacion de datos de instancia y de clase.<br>
 * Orden de inicializacion de variables cuando se crean objetos.<br>
 * Modificacion de variable statica desde un bloque no statico (de instancia) que se ejecuta por cada nueva instancia.<br>
 * Modificacion de variable statica desde constructor que se ejecuta por cada nueva instancia.<br>
 * Utilizacion de this para evitar conflicto con nombres de argumentos vs datos de instancia.<br>
 * Encapsulacion, creacion de metodos getters para datos de clase private que son actualizados desde constructor y 
 * bloque de inicializacion de instancia.<br>
 * Se requiere de una referencia para llamar a metodos de instancia.<br>
 * No se requiere de una referencia (sino del nombre de la Clase) para llamar a metodos de clase.<br>
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Inicializo {
    // declaracion e inicializacion de datos de clase
    private static int cantidadDeObjetos = 44;
    private static int valorInicial = 0;
    private static StringBuffer nombresDeObjetos;
    // declaracion e inicializacion de datos de instancia
    private int soyObjetoNumero = -1;
    // declaracion de dato de instancia
    private String nombre;
    // bloque de inicializacion de clase, se ejecuta una sola vez por clase (la primera vez que se hace referencia a ella)
    static {
        System.out.println("estoy en bloque de inicializacion static");
        cantidadDeObjetos = valorInicial;
        nombresDeObjetos = new StringBuffer("");
    }
    // bloque de inicializacion de instancia, se ejecuta una vez por cada objeto creado
    {
        System.out.println("estoy en bloque de inicializacion no static");
        soyObjetoNumero = ++cantidadDeObjetos;
    }
    /**
     * Constructor para objetos de clase Inicializo
     * Se ejecuta una vez por cada objeto Inicializo a crear.
     * No hay constructor por defecto o nulo.
     * Hay constructor especifico.
     */
    public Inicializo(String nombre)
    {
        System.out.println("estoy en constructor");
        this.nombre = nombre;
        nombresDeObjetos.append("\n" + nombre);
    }

    public static int getCantidadDeObjetos() {
        return cantidadDeObjetos;
    }
    public static String getNombresDeObjetos() {
        return nombresDeObjetos.toString();
    }
    public int getObjetoNumero() { 
        return soyObjetoNumero; 
    }
    public String getNombre() { 
        return nombre; 
    }
    public static void main(String[] args) {
        System.out.println("main():Incio Programa");
        System.out.println("main():Hay " + Inicializo.getCantidadDeObjetos() + " objetos creados y se llaman: " + Inicializo.getNombresDeObjetos());
        Inicializo i1 = new Inicializo("pepe");
        System.out.println("main():Creo obj Inicializo que se llama: " + i1.getNombre() + " y es el obj. numero: " + i1.getObjetoNumero());
        System.out.println("main():Hay " + Inicializo.getCantidadDeObjetos() + " objetos creados y se llaman: " + Inicializo.getNombresDeObjetos());
        Inicializo i2 = new Inicializo("juan");
        System.out.println("main():Creo obj Inicializo que se llama: " + i2.getNombre() + " y es el obj. numero: " + i2.getObjetoNumero());
        System.out.println("main():Hay " + Inicializo.getCantidadDeObjetos() + " objetos creados y se llaman: " + Inicializo.getNombresDeObjetos());
        Inicializo i3 = new Inicializo("ana");
        System.out.println("main():Creo obj Inicializo que se llama: " + i3.getNombre() + " y es el obj. numero: " + i3.getObjetoNumero());
        System.out.println("main():Hay " + Inicializo.getCantidadDeObjetos() + " objetos creados y se llaman: " + Inicializo.getNombresDeObjetos());
        System.out.println("main():Fin Programa");
    }
}
