import java.util.*;
/**
 * Este programa utiliza la clase Random del package java.util
 * Tambien utiliza la clase Math del package java.lang
 * Genera numeros aleatorios (al azar) utilizando la clase Ramdom y el metodo Math.random()
 * Usando el metodo nextInt(int n) de la clase Random, nos permite generar numeros aleatorios
 * entre 0 y n (exclusivo, es decir, retorna numeros enteros entre 0 y (n-1) inclusive).
 * Hay un arreglo de tipo String[] que mantiene "partes" de una posible frase, "armamos frases"
 * en forma aleatoria tomando estas "partes". El azar permitira armar frases con sentido y otras
 * sin sentido.
 * Para favorecer la creacion de frases con sentido, la segunda generacion de frases no permite 
 * elegir mas de una vez la misma "parte" de la frase, evitando su repeticion.
 * Se puede observar el uso de sequencia de escape \ para imprimir comillas dobles.
 * Se puede observar la sobrecarga del metodo setEstaUsado()
 * Se puede observar el manejo de arreglos y la propiedad length
 * Se puede observar el uso de la clausula import (debe estar al comienzo del programa)
 * <p>Math.random() devuelve un double entre 0.0 y 1.0, si quiero obtener a partir de este numero
 * al azar un numero entero entre 0 y 6 (resultado) podria hacer:</p>
 * double d = Math.random();<br>
 * d = d * 100000;<br>
 * int dint = (int) d;<br>
 * int resultado = dint % 7;<br><br>
 * <p>Tambien podria haberlo escrito como:</p>
 * int resultado = (((int) (Math.random() * 100000)) % 7);<br>
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class TextoAleatorio
{
	// instance variables - replace the example below with your own
	private String[] frase = new String[] {"juan ","ana ","invita a ",
	                                 "a salir ","al cine ","al bar ","a bailar "};
	private boolean[] estaUsado = new boolean[frase.length];

	/**
	 * Constructor para objectos de clase TextoAleatorio
	 * Genera numeros aleatorios que los utiliza para armar frases
	 */
	public TextoAleatorio()
	{
		// initialise instance variables
		Random r = new Random();
		int nfrase = 0;
		System.out.println("Genero frases I");
		for(int v = 0;v<20;v++) {
		    System.out.print("frase aleatoria " + v + ": \"");
    		for(int i = 0;i<4;i++) System.out.print(frase[r.nextInt(frase.length)]);   
    		System.out.println("\"");
    	}
		System.out.println("Genero frases II (no repito partes de la frase)");
		for(int v = 0;v<20;v++) {
		    // todas las partes de la frase no han sido usadas
		    setEstaUsado(false);
		    System.out.print("frase aleatoria " + v + ": \"");
    		for(int i = 0;i<4;i++) {
    		    nfrase = r.nextInt(frase.length);
    		    while ( getEstaUsado(nfrase) ) nfrase = r.nextInt(frase.length);
    		    System.out.print(frase[nfrase]);
    		    // indico esta parte de la frase como usada
    		    setEstaUsado(nfrase,true);
    		}
    		System.out.println("\"");
    	}
		System.out.println("Genero frases III usando Math.random()");
		for(int v = 0;v<20;v++) {
		    System.out.print("frase aleatoria " + v + ": \"");
    		for(int i = 0;i<4;i++) System.out.print(frase[ (((int) (Math.random() * 100000)) % frase.length) ] );   
    		System.out.println("\"");
    	}
    	
	}
	/**
	 * Inicio de esta Aplicacion, simplemente creo una instancia de esta clase
	 * 
	 * @param  args argumentos ingresados en la linea de comandos, parametros de la aplicacion
	 */
	public static void main(String[] args)
	{
		// put your code here
		new TextoAleatorio();
	}
	/**
	 * Indico si una parte de la frase ha sido usada o no
	 * 
	 * @param  numeroFrase Numero de parte de frase (entre 0 y (frase.length - 1))
	 * @param b indica si dicha parte de la frase fue usada o no
	 */
	public void setEstaUsado(int numeroFrase,boolean b) {
	    estaUsado[numeroFrase] = b;
	}
	/**
	 * Indico si todas las partes de la frase han sido usadas o no
	 * 
	 * @param b indica si todas las partes de la frase fueron usadas o no
	 */
	public void setEstaUsado(boolean b) {
	    for(int i = 0;i<frase.length;i++) setEstaUsado(i,b);
	}
	/**
	 * 
	 * @param numeroFrase Numero de parte de frase (entre 0 y (frase.length - 1))
	 * @return Devuelve true si la parte de la frase fue usada o false si no fue usada
	 */
	public boolean getEstaUsado(int numeroFrase) {
	    return estaUsado[numeroFrase];
	}
}
