/**
 * Sobreescritura de los metodos toString(), equals() y hashCode() de la clase Object
 * �Por que debo sobreescribirlos? Este programa muestra la implementacion de la clase
 * Cuenta que representa una Cuenta Corriente que tiene un titular, un numero (ambos
 * atributos son final) y un saldo.
 * Para saber si dos cuentas son iguales, me interesa tener en cuenta el numero de cuenta.
 * Para saber cual es el hashCode() de una cuenta (es decir, una forma univoca de representar
 * a traves de un entero a un objeto de clase cuenta) , me interesa que sea a traves del numero
 * de la cuenta corriente que es univoco.
 * Para tener una representacion en forma de String  de un objeto de clase cuenta, me gustaria
 * que fuese de la siguiente forma: "Cuenta N� xxxxx A Nombre de xxxxxxxxxxxxxxx con Saldo xxxxxxxxx"
 * Este programa implementa a Cuenta sin sobreescribir los metodos de la clase Object, a ver que
 * tanto se adaptan estos a nuestras necesidades.
 * 
 * @author G. Cherencio
 * @version 1.0
 */

public class Cuenta
{
    private final String titular;
    private final int numero;
    private float saldo;
    public static int ultimo_numero = 1000;
    
    public Cuenta(String titular) {
       this.titular = titular;
       numero = ++ultimo_numero;
       saldo = 0.0F;
    }
    public Cuenta(String titular, int numero)
    {
       this.titular = titular;
       this.numero = numero;
       if ( numero > ultimo_numero ) ultimo_numero = numero;
    }
    public Cuenta(String titular, int numero, float saldo)
    {
       this.titular = titular;
       this.numero = numero;
       this.saldo  = saldo;
       if ( numero > ultimo_numero ) ultimo_numero = numero;
    }

    public static void main(String[] args) {
        Cuenta c1, c2;
        c1 = new Cuenta("Pepe",10001);
        c2 = new Cuenta("Juan",10001,125.35F);
        // el metodo equals(Object o) da false en este caso porque los objs tienen distinto hashCode()
        if (c1.equals(c2)) System.out.println("Las cuentas son iguales");
        else               System.out.println("Las cuentas son distintas");
        System.out.println("A continuacion muestro los dos objetos Cuenta:");
        System.out.println("Usando el metodo toString() de la clase Object para objeto Cuenta (implicitamente):");
        System.out.println(c1);
        System.out.println(c2);
        System.out.println("Usando el metodo toString() de la clase Object para objeto Cuenta (explicitamente):");
        System.out.println(c1.toString());
        System.out.println(c2.toString());
        System.out.println("A continuacion muestro la identificacion de cada objeto");
        System.out.println("Usando el metodo hashCode() de la clase Object para objeto Cuenta:");
        System.out.println(c1.hashCode());
        System.out.println(c2.hashCode());
        System.out.println("�Esta conforme con el funcionamiento de los metodos toString(),equals() y hashCode()");
        System.out.println("de la clase Object?");
        System.out.println("Para solucionar esto debemos sobreescribir (Overriding) estos metodos y");
        System.out.println("hacer nuestra propia version de toString(),equals() y hashCode()");
        System.out.println("esto lo hacemos en la clase Cuenta2");
    }
}
