/**
 * Sobreescritura de los metodos toString(), equals() y hashCode() de la clase Object
 * Por que debo sobreescribirlos? Este programa muestra la implementacion de la clase
 * Cuenta que representa una Cuenta Corriente que tiene un titular, un numero (ambos
 * atributos son final) y un saldo.
 * Para saber si dos cuentas son iguales, me interesa tener en cuenta el numero de cuenta.
 * Para saber cual es el hashCode() de una cuenta (es decir, una forma univoca de representar
 * a traves de un entero a un objeto de clase cuenta) , me interesa que sea a traves del numero
 * de la cuenta corriente que es univoco.
 * Para tener una representacion en forma de String  de un objeto de clase cuenta, me gustaria
 * que fuese de la siguiente forma: "Cuenta N xxxxx A Nombre de xxxxxxxxxxxxxxx con Saldo xxxxxxxxx"
 * Este programa implementa a Cuenta sobreescribiendo los metodos de la clase Object para adaptarlos
 * a nuestras necesidades.
 *
 * 
 * @author G. Cherencio
 * @version 1.0
 */

public class Cuenta3
{
    private final String titular;
    private final int numero;
    private float saldo;
    public static int ultimo_numero = 1000;
    
    public Cuenta3(String titular) {
       this.titular = titular;
       numero = ++ultimo_numero;
       saldo = 0.0F;
    }
    public Cuenta3(String titular, int numero)
    {
       this.titular = titular;
       this.numero = numero;
       if ( numero > ultimo_numero ) ultimo_numero = numero;
    }
    public Cuenta3(String titular, int numero, float saldo)
    {
       this.titular = titular;
       this.numero = numero;
       this.saldo  = saldo;
       if ( numero > ultimo_numero ) ultimo_numero = numero;
    }
    /**
     * Sobreescribo el metodo hashCode() de la clase Object
     * 
     */
    public int hashCode() {
        return numero;
    }
    /**
     * Sobreescribo el metodo equals() de la clase Object?
     * NO, lo estoy sobrecargando!!
     */
    public boolean equals(Cuenta3 c) {
       return (numero == c.numero);
    }
    /**
     * Sobreescribo el metodo toString() de la clase Object
     * 
     */
    public String toString() {
       return "Cuenta N " + numero + " A Nombre de " + titular + " con Saldo " + saldo;        
    }

    public static void main(String[] args) {
        Cuenta3 c1, c2;
        c1 = new Cuenta3("Pepe",10001);
        c2 = new Cuenta3("Juan",10001,125.35F);
        // aqui llamo a mi version de equals(Cuenta c)
        if (c1.equals(c2)) System.out.println("Las cuentas son iguales");
        else               System.out.println("Las cuentas son distintas");
        // aqui llamo a la version de la clase Object de equals(Object c)
        if (c1.equals((Object) c2)) System.out.println("Las cuentas son iguales");
        else                        System.out.println("Las cuentas son distintas");
        System.out.println("A continuacion muestro los dos objetos Cuenta:");
        System.out.println("Usando el metodo toString() de la clase Object para objeto Cuenta (implicitamente):");
        System.out.println(c1);
        System.out.println(c2);
        System.out.println("Usando el metodo toString() de la clase Object para objeto Cuenta (explicitamente):");
        System.out.println(c1.toString());
        System.out.println(c2.toString());
        System.out.println("A continuacion muestro la identificacion de cada objeto");
        System.out.println("Usando el metodo hashCode() de la clase Object para objeto Cuenta:");
        System.out.println(c1.hashCode());
        System.out.println(c2.hashCode());
    }
}
